# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Northwestern University.
#
# invenio-analytics-importer is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Config."""

ANALYTICS_IMPORTER_MATOMO_URL = ""
"""Your Matamo site's base url."""

ANALYTICS_IMPORTER_MATOMO_SITE_ID = ""
"""Your Matamo site's id."""

ANALYTICS_IMPORTER_MATOMO_TOKEN = ""
"""A Matamoto application token."""
